@@ -18,7 +18,7 @@
 #endif
 #endif
 
-#if defined(WIN32) && !defined(__CYGWIN__)
+#if defined(WIN32) && !defined(__CYGWIN__) && !defined(__MINGW32__)
 #include "win32config.h"
 #elif defined(macintosh)
 #include "config-mac.h"
